<?php

class RecordBookData {
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var integer */
    public $planID;
    /** @var string */
    public $planExternalID;
    /** @var integer */
    public $studentID;
    /** @var string */
    public $studentExternalID;
    /** @var integer */
    public $facultyID;
    /** @var string */
    public $facultyExternalID;
    /** @var string */
    public $status;
    /** @var string */
    public $degree;
    /** @var string */
    public $form;
    /** @var string */
    public $speciality;
    /** @var integer */
    public $grade;
    /** @var integer */
    public $group;
    /** @var integer */
    public $groupID;
}

class StudentData {
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var string */
    public $firstName;
    /** @var string */
    public $secondName;
    /** @var string */
    public $lastName;
    /** @var RecordBookData[] */
    public $recordBooks;
}

class Controller_Api_V0_Student extends Controller_Handler_Api {
    /** Нормализация данных зачетки
     * @param RecordBookData $data
     * @return RecordBookData
     */
    private function normalizeRecordBookData($data) {
        if (!isset($data->id) && isset($data->externalID)) {
            $data->id = Model_RecordBook::withExternalID($data->externalID);
        }

        if (!isset($data->studentID)) {
            if (isset($data->studentExternalID)) {
                $data->studentID = Model_Student::withExternalID($data->studentExternalID);
                if ($data->studentID === -1) {
                    throw new InvalidArgumentException('RecordBook student not found');
                }
            } else {
                throw new InvalidArgumentException('RecordBook without student');
            }
        }

        if (!isset($data->planID)) {
            if (isset($data->planExternalID)) {
                $data->planID = Model_Plan::withExternalID($data->planExternalID);
                if ($data->planID == -1) {
                    throw new InvalidArgumentException('RecordBook plan not found');
                }
            } else {
                throw new InvalidArgumentException('RecordBook without plan');
            }
        }

        if (!isset($data->facultyID)) {
            if (isset($data->facultyExternalID)) {
                $facultyID = $this->getFacultyIDByExternalID($data->facultyExternalID);
                if (is_null($facultyID)) {
                    throw new InvalidArgumentException('RecordBook faculty not found');
                }
                $data->facultyID = $facultyID;
            } else {
                throw new InvalidArgumentException('RecordBook without faculty');
            }
        }

        switch ($data->degree) {
            case 'Академический бакалавр':
            case 'Бакалавр':
                $data->degree = 'bachelor';
                break;

            case 'Специалист':
                $data->degree = 'specialist';
                break;

            case 'Магистр':
                $data->degree = 'master';
                break;

            case 'Аспирант':
//                $data->degree = 'postgraduate';
//                break;
                throw new InvalidArgumentException('Bad recordBook degree');

            default:
                throw new InvalidArgumentException('Bad recordBook degree');
        }

        switch ($data->status) {
            case 'Учится':
                $data->status = 'common';
                break;

            case 'Академ':
                $data->status = 'outlet';
                break;

            case 'Выпустился':
//                $data->status = 'expulsion';
//                break;
                throw new InvalidArgumentException('Bad recordBook status');

            case 'Отчислен':
//                $data->status = 'leave';
//                break;
                throw new InvalidArgumentException('Bad recordBook status');

            default:
                throw new InvalidArgumentException('Bad recordBook status');
        }

        switch ($data->form) {
            case 'Очная':
                $data->form = '1';
                break;

            case 'Очно-заочная':
//                $data->form = '2';
//                break;
                throw new InvalidArgumentException('Bad recordBook form');

            case 'Заочная':
//                $data->form = '3';
//                break;
                throw new InvalidArgumentException('Bad recordBook form');

            default:
                throw new InvalidArgumentException('Bad recordBook form');
        }

        if (!isset($data->groupID)) {
            if (isset($data->grade) && isset($data->degree)) {
                $gradeID = Model_Grades::getID($data->grade, $data->degree);
                if ($gradeID == -1) {
                    throw new InvalidArgumentException('RecordBook grade not found');
                }

                $group = Model_Group::find($gradeID, $data->group, $data->facultyID);
                if (is_null($group)) {
                    throw new InvalidArgumentException('RecordBook group not found');
                }

                $data->groupID = $group->getID();
            } else {
                throw new InvalidArgumentException('RecordBook without grade or grade');
            }
        }

        return $data;
    }

    /** Нормализация данных студента
     * @param StudentData $data
     * @return StudentData
     */
    private function normalizeStudentData($data, $facultyID) {
        if (!isset($data->id) && isset($data->externalID)) {
            $data->id = Model_Student::withExternalID($data->externalID);
        }

        if (!isset($data->id) || ($data->id == -1)) {
            $foundList = Model_Students::searchStudentsByName([$data->firstName, $data->secondName, $data->lastName], $facultyID);
            $foundSet = [];
            foreach ($foundList as $item) {
                if (!in_array($item['ID'], $foundSet)) {
                    $foundSet[] = $item['ID'];
                }
            }
            if (count($foundSet) === 1) {
                $data->id = (int)$foundList[0];
            } elseif (count($foundList) > 1) {
                throw new InvalidArgumentException('Duplicate student');
            }
        }

        foreach ($data->recordBooks as &$recordBook) {
            if (isset($data->id)) {
                $recordBook->studentID = $data->id;
            }

            if (isset($data->externalID)) {
                $recordBook->studentExternalID = $data->externalID;
            }

            try {
                $recordBook = $this->normalizeRecordBookData($recordBook);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_STUDENTS',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($recordBook),
                ));
                $recordBook = null;
            }
        }

        return $data;
    }

    /**
     * @param RecordBookData $data
     * @return integer[]
     */
    private function createOrUpdateRecordBook($data) {
        $needUpdate = isset($data->id) && ($data->id > 0);

        try {
            $recordBookBuilder = Model_RecordBook::make()
                ->studentID($data->studentID)
                ->planID($data->planID)
                ->semesterID(Model_System::$SemesterID)
                ->group($data->groupID)
                ->externalID($data->externalID);

            if ($needUpdate) {
                $recordBook = Model_RecordBook::load($data->id);
                $recordBook->update($recordBookBuilder->getPreparedData());
            } else {
                $recordBook = $recordBookBuilder->create();
            }
        } catch (InvalidArgumentException $e) {
            $this->badRequestError($this->makeErrorMsg($data, $e->getMessage()));
        }

        return ['ID' => $recordBook->ID];
    }

    /**
     * @param StudentData $data
     * @return integer[]
     */
    private function createOrUpdateStudent($data) {
        $needUpdate = isset($data->id) && ($data->id > 0);
        $rbList = [];

        try {
            if ($needUpdate) {
                $student = Model_Student::load($data->id);
                if (!$student->update())
                    throw new Database_Exception('Student change info failed!');
            } else {
                $student = Model_Student::make()->externalID($data->externalID)
                    ->firstName($data->firstName)
                    ->secondName($data->secondName)
                    ->lastName($data->lastName)
                    ->create();
            }

            foreach ($data->recordBooks as $recordBookData) {
                $recordBookData->studentID = $student->ID;
                $rbList[] = $this->createOrUpdateRecordBook($recordBookData);
            }
        } catch (InvalidArgumentException $e) {
            $this->badRequestError($this->makeErrorMsg($data, $e->getMessage()));
        }

        return ['ID' => $student->ID, 'RecordBooks' => $rbList];
    }

//    public function action_get_index() {
//        $this->response->body('{ what: get index }');
//
//        return '{ what: get index }'; //$this->response;
//    }

//    /**
//     * @api {put} api/v0/student/recordBook Add new record book(s)
//     * @apiName Add new record book
//     * @apiGroup Record books
//     * @apiVersion 0.1.2
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} externalID
//     * @apiParam {String} studentExternalID
//     * @apiParam {String} planExternalID
//     * @apiParam {String} facultyExternalID
//     * @apiParam {String} status
//     * @apiParam {String} level
//     * @apiParam {String} form
//     * @apiParam {String} speciality
//     * @apiParam {Int} grade
//     * @apiParam {Int} group
//     */
//    public function action_put_recordBook() {
//        try {
//            if ($this->request->query('batch') !== NULL) {
//                $data = json_decode($this->request->body());
//
//                $res = [];
//                foreach ($data as $item) {
//                    $data = $this->normalizeRecordBookData($item);
//                    $res[] = $this->createRecordBook($data);
//                }
//            } else {
//                /** @var RecordBookData $data */
//                $data = (object)$this->request->query();
//
//                $data = $this->normalizeRecordBookData($data);
//                $res[] = $this->createRecordBook($data);
//            }
//        } catch (Exception $e) {
//            $this->badRequestError($e->getMessage());
//        }
//
//        return $res;
//    }

//    /**
//     * @api {put} api/v0/student Add new student(s)
//     * @apiName Add new student
//     * @apiGroup Student
//     * @apiVersion 0.1.0
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} firstName
//     * @apiParam {String} secondName
//     * @apiParam {String} lastName
//     * @apiParam {String} departmentName
//     * @apiParam {String} [departmentID]
//     * @apiParam {String} jobPositionName
//     * @apiParam {String} [jobPositionID]
//     * @apiParam {String} status
//     */
    public function action_put_index() {
        $res = [];
        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());
                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

                foreach ($data->students as $student) {
                    try {
                        $studentData = $this->normalizeStudentData($student, $facultyID);

                        $res[] = $this->createOrUpdateStudent($studentData);
                    } catch (Exception $e) {
                        Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                            '{0}' => 'SYNC_STUDENTS',
                            '{1}' => $e->getMessage(),
                            '{2}' => json_encode($student),
                        ));
                    }
                }
            } else {
//                $data = (object)$this->request->query();
//
//                $studentData = $this->normalizeStudentData($data);
//                $res[] = $this->createOrUpdateStudent($studentData);
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

//    public function action_get_list_by_name() {
//        return '{ what: nothing }';
//    }
}
